/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2005, Micromuse, Ltd.                     */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* Log()
*
* Log a message to the installer log file.
*
* @param level The lowest level at which the message should be logged.
*              0 - Errors
*              1 - Warnings
*              2 - Info
*              3 - Debug
*
* @param msg The message to be logged.
*
********************************************************************************
*/
function Log(level, msg)
{
    try
    {
        var logger = Components.classes["@micromuse.com/NCILog;1"].
            getService(Components.interfaces.nsINCILog);

        switch (level)
        {
            case 0:
                logger.error(msg);
                break;
            case 1:
                logger.warning(msg);
                break;
            case 2:
                logger.info(msg);
                break;
            case 3:
                logger.debug(msg);
                break;
            default:
                break;
        }
    }
    catch(e)
    {
        // Logging failure must not affect functionality, so fail silently.
    }
}

/**
********************************************************************************
*
* GetNcHome()
*
* Get the value of the NCHOME environment variable.
*
* @return The value of the NCHOME environment variable.
*
********************************************************************************
*/
function GetNcHome()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    return propMgr.GetProp("$NCHOME").value;
}

/**
********************************************************************************
*
* ReadFile
*
* Read an existing file.  The caller should ensure the file exists before
* calling this function.
*
* @param file The nsIFile object to be read from.
*
* @return The entire file contents as a string.
*
********************************************************************************
*/
function ReadFile(file)
{
    var result = "";

    var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].
        createInstance(Components.interfaces.nsIScriptableInputStream);

    try
    {
        fstream.init(file, 1, 0, 0);
        sstream.init(fstream);

        var bytes = fstream.available();
        while (bytes > 0)
        {
            result += sstream.read(bytes);

            bytes = fstream.available();
        }
        sstream.close();
        fstream.close();
    }
    catch(e)
    {
        Log(0, e);
    }

    return result;
}

/**
********************************************************************************
*
* GetFullPlatformPath()
*
* Convert the stub file paths found in our filelist files to a full path that is
* correct for the current platform.  On Windows, this will also involve
* replacing forward slashes with backslashes.
*
* For example, if $NCHOME is set to /opt/netcool then for an input of
* precision/aoc/default/Device.aoc, this function will return
* /opt/netcool/precision/aoc/default/Device.aoc.
*
* @param relocName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetFullPlatformPath(relocName)
{
    var result = GetNcHome() + "/" + relocName;

    if (ncipackage.shortArch == "win32")
    {
        result = result.replace(/\//g, "\\");
    }

    return result;
}

/**
********************************************************************************
*
* GetFullPlatformPath()
*
* Given a stub file path (like the ones found in our filelist files) for a
* default config file, return the full path where the used version of the config
* file should go.  This will be correct for the current platform, i.e. on
* Windows, forward slashes will be replaced with backslashes.
*
* For example, if $NCHOME is set to /opt/netcool then for an input of
* precision/aoc/default/Device.aoc, this function will return
* /opt/netcool/precision/aoc/Device.aoc.
*
* @param relocName Stub path name to expand and make platform specific.
*
* @return The expanded, platform specific path name.
*
********************************************************************************
*/
function GetFullInstalledPath(relocName)
{
    return GetFullPlatformPath(relocName.replace(/default\//, ""));
}

/**
********************************************************************************
*
* GetSignature()
*
* Get a signature for a file.  This can be used to tell if a file has changed.
* The signature consists of the length of the file, followed by a hash code
* computed from the contents of the file.
*
* @param fileName Absolute path to the file to get the signature for.
*
* @return The signature for the given file.
*
********************************************************************************
*/
function GetSignature(fileName)
{
    var answer = "";

    var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(fileName);

    if (file.exists())
    {
        answer += file.fileSize;

        var contents = ReadFile(file);

        var highBits = 15 << 28;
        var tmp = 0;
        var hash = 0;
        for (var count = 0; count < contents.length; ++count)
        {
            hash <<= 4;
            hash += contents.charCodeAt(count);
			tmp = hash & highBits;
			if (tmp != 0)
			{
				hash = (hash ^ (tmp >>> 24)) & ~highBits;
			}
        }

        answer += "#" + hash;

        Log(3, "Signature for file " + fileName + " is " + answer);
    }
    else
    {
        answer = "0#0"
    }

    return answer;
}

/**
********************************************************************************
*
* DoFilesDiffer()
*
* Check whether two files differ.  Any differences, including whitespace, will
* be treated as a difference.
*
* If the first file doesn't exist, then this is NOT counted as a difference.
*
* If the second file doesn't exist, but the first one does, then this IS counted
* as a difference.
*
* @param fileNameA Absolute path to the first file to compare.
*
* @param fileNameB Absolute path to the second file to compare.
*
* @return true if both files exist and are different; otherwise false.
*
********************************************************************************
*/
function DoFilesDiffer(fileNameA, fileNameB)
{
    var answer = false;

    var fileA = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    fileA.initWithPath(fileNameA);

    if (fileA.exists())
    {
        var fileB = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
        fileB.initWithPath(fileNameB);

        if (fileB.exists())
        {
            var contentsA = ReadFile(fileA);
            var contentsB = ReadFile(fileB);

            if (contentsA != contentsB)
            {
                Log(3, fileNameA + " and " + fileNameB + " are different");

                answer = true;
            }
            else
            {
                Log(3, fileNameA + " and " + fileNameB + " are the same");
            }
        }
        else
        {
            Log(3, fileNameB + " does not exist");

            answer = true;
        }
    }
    else
    {
        Log(3, fileNameA + " does not exist");
    }

    return answer;
}

/**
********************************************************************************
*
* ConfigFilePreCheck()
*
* Work through the list of config files, checking whether any of them are
* different from their default versions.  If so, create a file called
* $NCHOME/package_name.user.tmp containing the array indexes of the changed
* files.
*
* Additionally, create a file containing signatures of the current default
* config files.  This is used to ensure the user is only warned when this
* installation actually changes a file they have changed themselves.
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
********************************************************************************
*/
function ConfigFilePreCheck(configFiles)
{
    var userFilename = GetFullPlatformPath(ncipackage.name + ".user.tmp");
    var signatureFilename = GetFullPlatformPath(ncipackage.name + ".signature.tmp");

    var userContents = "";
    var signatureContents = "";

    try
    {
        var userFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        userFile.initWithPath(userFilename);

        var signatureFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        signatureFile.initWithPath(signatureFilename);

        var userStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        userStream.init(userFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

        var signatureStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        signatureStream.init(signatureFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

        var installedFileName = "";
        var defaultFileName = "";

        for (var count = 0; count < configFiles.length; ++count)
        {
            installedFileName = GetFullInstalledPath(configFiles[count]);
            defaultFileName = GetFullPlatformPath(configFiles[count]);

            //
            // Get the length and hash code for the current default file.
            //
            if (signatureContents.length > 0)
            {
                signatureContents += ",";
            }
            signatureContents += GetSignature(defaultFileName);

            //
            // Has the user modified the file?
            //
            if (DoFilesDiffer(installedFileName, defaultFileName))
            {
                if (userContents.length > 0)
                {
                    userContents += ",";
                }
                userContents += count;
            }
        }

        userStream.write(userContents, userContents.length);
        userStream.close();

        signatureStream.write(signatureContents, signatureContents.length);
        signatureStream.close();
    }
    catch(e)
    {
        Log(0, e);
        Log(0, "Unable to write " + userFilename + " and " + signatureFilename);
    }
}

/**
********************************************************************************
*
* PopulateSignatureArray()
*
* Populate an array with the signatures of the previous config files.  The
* returned array is indexed using the index of the config file array.
*
* @param file The file to read that contains the signatures of the previous
*             config files.
*
* @return The populated array of previous config signatures.
*
********************************************************************************
*/
function PopulateSignatureArray(file)
{
    var result = [];

    var contents = ReadFile(file);
    if (contents.length > 0)
    {
        result = contents.split(",");
    }

    return result;
}

/**
********************************************************************************
*
* PopulateChangedArray()
*
* Populate an array with the indexes of those config files that have changed.
* The returned array is indexed using the index of the changed config files that
* are read from the given file.
*
* @param file The file to read that contains the indexes of the changed config
*             files.
*
* @return The populated array of changed indexes.
*
********************************************************************************
*/
function PopulateChangedArray(file)
{
    var result = new Array;

    var contents = ReadFile(file);
    if (contents.length > 0)
    {
        var contentsArray = contents.split(",");

        for (var count = 0; count < contentsArray.length; ++count)
        {
            result[Number(contentsArray[count])] = true;
        }
    }

    return result;
}

/**
********************************************************************************
*
* OverwriteFile()
*
* Overwrite one file with the contents of another.
*
* @param targetFileName Name of file to overwrite.
*
* @param sourceFileName Name of source file.
*
********************************************************************************
*/
function OverwriteFile(targetFileName, sourceFileName)
{
    var sourceFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    sourceFile.initWithPath(sourceFileName);

    try
    {
        if (sourceFile.exists())
        {
            var contents = ReadFile(sourceFile);

            var targetFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            targetFile.initWithPath(targetFileName);

            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(targetFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
            foStream.write(contents, contents.length);
            foStream.close();
        }
        else
        {
            Log(0, "Cannot update file " + targetFileName +
                   " because " + sourceFileName + " does not exist");
        }
    }
    catch(e)
    {
        Log(0, "Unable to write to file " + targetFileName);
    }
}

/**
********************************************************************************
*
* ConfigFilePostInstall()
*
* Work through the list of config files, checking whether the default was
* different to the installed file before installation.  If so, inform the user.
* If not, copy the default file to the install directory.
*
* @param configFiles An array (which may be empty) containing a list of the
*                    configuration files associated with the current package.
*
********************************************************************************
*/
function ConfigFilePostInstall(configFiles)
{
    var userFilename = GetFullPlatformPath(ncipackage.name + ".user.tmp");
    var signatureFilename = GetFullPlatformPath(ncipackage.name + ".signature.tmp");

    try
    {
        var userFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        userFile.initWithPath(userFilename);

        var changedArray = [];

        if (userFile.exists())
        {
            changedArray = PopulateChangedArray(userFile);

            userFile.remove(false);
        }

        var signatureFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        signatureFile.initWithPath(signatureFilename);

        var signatures = [];

        if (signatureFile.exists())
        {
            signatures = PopulateSignatureArray(signatureFile);

            signatureFile.remove(false);
        }

        var copyIt = true;
        var installedFile = "";
        var defaultFile = "";

        for (var count = 0; count < configFiles.length; ++count)
        {
            installedFile = GetFullInstalledPath(configFiles[count]);
            defaultFile = GetFullPlatformPath(configFiles[count]);

            copyIt = true;

            if (changedArray[count])
            {
                //
                // File may have been changed by the user.
                // If so, tell the user to merge it themselves.
                // But first double check that the newly
                // installed default is different to the
                // existing version.
                //
                if (DoFilesDiffer(installedFile, defaultFile))
                {
                    copyIt = false;

                    //
                    // The next possibility is that although
                    // the user has changed the file, this install
                    // isn't changing the default version.  If this
                    // is the case then we won't copy the default,
                    // but the user doesn't need to be warned.
                    //
                    if (signatures[count] != GetSignature(defaultFile))
                    {
                        Log(1, defaultFile + " has changed, and " + installedFile
                             + " contains customisations not present in it"
                             + " - please merge any required changes by hand");
                    }
                    else
                    {
                        Log(2, installedFile + " contains customisations not present in "
                             + defaultFile + ", but no changes were made to " + defaultFile
                             + " on this installation");
                    }
                }
            }

            if (copyIt)
            {
                //
                // File has not been changed by the user.
                // Copy the default to the install directory.
                //
                OverwriteFile(installedFile, defaultFile);
            }
        }
    }
    catch(e)
    {
        Log(0, e);
    }
}

