/**
 * Post-install script to configure the config files.
 *
 * Note on exception handling:
 * All exceptions are caught and logged in this main function. Other functions
 * shouldn't be catching exceptions so that this function does not carry on
 * with the post-install actions if an error occurs (which otherwise could
 * mess things up).
 */
function postInstall()
{
	log( 2, "Starting webtop postInstall()..." );

	try
	{
		// --- Configure the ncwDataSourceDefinitions.xml file ---
		var datasrcDefFilename = getNchome() +
			"/etc/webtop/datasources/ncwDataSourceDefinitions.xml";

		log( 2, "Configuring " + datasrcDefFilename );

		var defFileContents = readFromFile( datasrcDefFilename );

		var newDefFileContents = processDefFile( defFileContents );
		writeToFile( datasrcDefFilename, newDefFileContents, true );

		log( 3, datasrcDefFilename + " configuration completed." );

		// --- Configure the server.init file ---
		var serverInitFilename = getNchome() + "/etc/webtop/server.init";

		log( 2, "Configuring " + serverInitFilename );

		var initFileContents = readFromFile( serverInitFilename );

		var newInitFileContents = processInitFile( initFileContents );
		writeToFile( serverInitFilename, newInitFileContents, true );

		log( 3, serverInitFilename + " configuration completed." );

		// --- Configure the Ant script (for PSML file registration) ---
		var antScript = getNchome() +
			"/etc/webtop/installer/machant_webtop.xml";

		log( 2, "Configuring " + antScript );

		var antScriptContents = readFromFile( antScript );
		var newAntScriptContents = processAntScript( antScriptContents );

		var newAntScript = getNchome() +
			"/etc/guifoundation/antonce/machant_webtop.xml";
		createFolders( getNchome() + "/etc/guifoundation/antonce/" );

		writeToFile( newAntScript , newAntScriptContents, false );

		log( 3, "machant_webtop.xml configuration completed." );
		
		log( 2, "Registering Webtop custom NGF viewpoints");
		registerNCWCustomViewpoints();
		log( 3, "Completed registration of Webtop viewpoints");
	}
	catch ( e )
	{
		log( 0, e );
		log( 0, "webtop postInstall() failed to complete." );

		return;
	}

	log( 3, "webtop postInstall() completed successfully." );
}

/**
 * Process the content of the ncwDataSourceDefinitions file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processDefFile( fileContents )
{
	log( 2, "Processing definitions file..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
		getService( Components.interfaces.nsINCIProperties );

	// Replace ObjectServer name (in 2 places).
	var regex = /(ncwDataSourceEntry name=").*?(")/g;
	var newContents = fileContents.replace( regex,
		"$1" + propMgr.GetProp( "NCWDATASOURCE0.NAME" ).value + "$2" ); 

	regex = /(ncwDataSourceDefinition .*?name=").*?(")/g;
	newContents = newContents.replace( regex,
		"$1" + propMgr.GetProp( "NCWDATASOURCE0.NAME" ).value + "$2" ); 

	// Replace ObjectServer host.
	regex = /(ncwOSConnection host=").*?(")/g;
	newContents = newContents.replace( regex,
		"$1" + propMgr.GetProp( "OBJECTSERVER0.HOST" ).value + "$2" );

	// Replace ObjectServer port.
	regex = /(ncwOSConnection.*?port=").*?(")/g;
	newContents = newContents.replace( regex,
		"$1" + propMgr.GetProp( "OBJECTSERVER0.PORT" ).value + "$2" );

	// Replace ObjectServer username.
	regex = /(ncwDataSourceCredentials.*?userName=").*?(")/g
	newContents = newContents.replace( regex,
		"$1" + propMgr.GetProp( "OBJECTSERVER0.USERNAME" ).value + "$2");

	// Replace ObjectServer password.
	regex = /(ncwDataSourceCredentials.*?password=").*?(")/g
	newContents = newContents.replace( regex,
		"$1" + propMgr.GetProp( "OBJECTSERVER0.PASSWORD" ).value + "$2");

	log( 3, "Definitions file processing completed." );

	return newContents;
}

/**
 * Process the content of the server.init file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processInitFile( fileContents )
{
	log( 2, "Processing server.init file..." );

	// Read and extract license information from GUI Foundation's license file.
	// NOTE: In actual production, it will be available from NCIProperties.
	var guifLicFile = getNchome() +
		"/guifoundation/conf/license.properties";
	var licFileContent = readFromFile( guifLicFile );

	var licRegex = /license\.server:(.*)/;
	var regexArray = licRegex.exec( licFileContent );
	var licInfo = regexArray[1];

	log( 3, "License host = " + licInfo );

	// Replace license.file property value.
	var regex = /(license\.file:).*/;
	var newContents = fileContents.replace( regex, "$1" + licInfo );

	log( 3, "server.init file processing completed." );

	return newContents;
}

/**
 * Register Webtop custom actions with the NGF
 *
 */
function registerNCWCustomViewpoints ( )
{
	var turbineResourceProps = getNchome() + 
		"/guifoundation/webapps/desktop/WEB-INF/conf/TurbineResources.properties";
	
	log( 3, "TurbineResource file is "+turbineResourceProps);
	
	var turbineResourceContents = readFromFile( turbineResourceProps );
	
	var ncwPackageString = "com.micromuse.ncw.modules";
	
	if(turbineResourceContents.indexOf(ncwPackageString) >= 0)
	{
		log( 3,"located " + ncwPackageString + "statement in "+turbineResourceProps);
	}
	else
	{
		log( 3,ncwPackageString + " not located in "+turbineResourceProps);
		if( ncipackage.shortArch == "win32" )
		{
			turbineResourceContents = turbineResourceContents + "\r\nmodule.packages=com.micromuse.ncw.modules\r\n";
			log( 3, "Updated turbine properties with ncw modules, windows line endings");
		}
		else
		{
			turbineResourceContents = turbineResourceContents + "\nmodule.packages=com.micromuse.ncw.modules\n";
			log( 3, "Updated turbine properties with ncw modules, unix line endings");
		}
	}
	
	writeToFile( turbineResourceProps, turbineResourceContents, true );
	
	return;
}

/**
 * Process the content of the Ant script.
 * <p>
 * The ant script contains the following variables:
 * <ul>
 * <li>__GUIFOUNDATION_WEBSERVER_PORT__</li>
 * <li>__NCSM0_ADMIN_USERNAME__</li>
 * <li>__NCSM0_ADMIN_PASSWORD__</li>
 * <li>__NCHOME__</li>
 * <li>__GUIFOUNDATION__</li>
 * </ul>
 *
 * @param txt (String) The content to process.
 * @return (String) The processed content.
 */
function processAntScript( fileContents )
{
	log( 2, "Processing Ant script..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
		getService(Components.interfaces.nsINCIProperties);

	var mainHttpServerPort =
		propMgr.GetProp("GUIFOUNDATION.WEBSERVER.PORT").value;
	var mainHttpServerUsername =
		propMgr.GetProp("NCSM0.ADMIN.USERNAME").value;
	var mainHttpServerPassword =
		propMgr.GetProp("NCSM0.ADMIN.PASSWORD").value;
	var nchome = getNchome();
	var guifoundation = nchome + "/guifoundation";


	var newContents = fileContents.replace( /__GUIFOUNDATION_WEBSERVER_PORT__/g,
		mainHttpServerPort );

	newContents = newContents.replace( /__NCSM0_ADMIN_USERNAME__/g,
		mainHttpServerUsername );

	newContents = newContents.replace( /__NCSM0_ADMIN_PASSWORD__/g,
		mainHttpServerPassword );

	newContents = newContents.replace( /__NCHOME__/g, nchome );

	newContents = newContents.replace( /__GUIFOUNDATION__/g,
		guifoundation );

	log( 3, "Ant script processing completed." );

	return newContents;
}
